#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/nfsserver-update.sh - update or generate new nfsserver
#                                             configuration
#
# Copyright (c) 2001-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2004-07-28 jed
# Last Update:  $Id: nfsserver-update.sh 73445 2020-03-24 11:54:19Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# read eislib etc.
. /var/install/include/eislib
. /var/install/include/configlib
. /var/install/include/jedlib

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2>/tmp/$(basename ${0})-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

#------------------------------------------------------------------------------
# rename variables
#------------------------------------------------------------------------------
rename_variables ()
{
    local _rv_renamed
    local _rv_output_file

    _rv_renamed=0
    _rv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_rv_renamed} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "renaming parameter(s) ..."
        cat ${_rv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for renamed parameter(s)!"
    fi

    rm -f ${_rv_output_file}
}

#------------------------------------------------------------------------------
# modify variables
#------------------------------------------------------------------------------
modify_variables ()
{
    local _mv_modified
    local _mv_output_file

    _mv_modified=0
    _mv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_mv_modified} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "modifying parameter(s) ..."
        cat ${_mv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for modified parameter(s)!"
    fi

    rm -f ${_mv_output_file}
}

#------------------------------------------------------------------------------
# add variables
#------------------------------------------------------------------------------
add_variables ()
{
    local _av_added
    local _av_output_file
    local _av_imax
    local _av_idx
    local _av_nfs_hosts_n
    local _av_jmax
    local _av_jdx

    _av_added=0
    _av_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    # v1.1.2
    if [ ${NFSSERVER_SHARE_N} -eq 0 ]
    then
        _av_imax=1
    else
        _av_imax=${NFSSERVER_SHARE_N}
    fi

    _av_idx=1
    while [ ${_av_idx} -le ${_av_imax} ]
    do
        eval _av_nfs_hosts_n='$NFSSERVER_SHARE_'${_av_idx}'_HOSTS_N'

        if [ ${_av_nfs_hosts_n} -eq 0 ]
        then
            _av_jmax=1
        else
            _av_jmax=${_av_nfs_hosts_n}
        fi

        _av_jdx=1
        while [ ${_av_jdx} -le ${_av_jmax} ]
        do
            # v1.1.2
            if [ -z "`grep \"^NFSSERVER_SHARE_${_av_idx}_HOSTS_${_av_jdx}_OPTION\" ${source_conf}`" ]
            then
                # optional parameter!
              # echo "- NFSSERVER_SHARE_${_av_idx}_HOSTS_${_av_jdx}_OPTION=''" >> ${_av_output_file}
                eval "NFSSERVER_SHARE_${_av_idx}_HOSTS_${_av_jdx}_OPTION=''"
              # _av_added=1
            fi

            _av_jdx=`expr ${_av_jdx} + 1`
        done

        # v1.3.13
        if [ -z "`grep ^NFSSERVER_SHARE_${_av_idx}_NAME ${source_conf}`" ]
        then
            echo "- NFSSERVER_SHARE_${_av_idx}_NAME=''" >> ${_av_output_file}
            eval "NFSSERVER_SHARE_${_av_idx}_NAME=''"
            _av_added=1
        fi

        _av_idx=`expr ${_av_idx} + 1`
    done

    # v1.3.12
    if [ -z "`grep ^NFSSERVER_SUPPORT_NFS3 ${source_conf}`" ]
    then
        echo "- NFSSERVER_SUPPORT_NFS3='yes'" >> ${_av_output_file}
        NFSSERVER_SUPPORT_NFS3='yes'
        _av_added=1
    fi

    if [ -z "`grep ^NFSSERVER_SUPPORT_NFS4 ${source_conf}`" ]
    then
        echo "- NFSSERVER_SUPPORT_NFS4='yes'" >> ${_av_output_file}
        NFSSERVER_SUPPORT_NFS4='yes'
        _av_added=1
    fi

    if [ ${_av_added} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "adding new parameter(s) ..."
        cat ${_av_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for added parameter(s)!"
    fi

    rm -f ${_av_output_file}
}

#------------------------------------------------------------------------------
# delete variables
#------------------------------------------------------------------------------
delete_variables ()
{
    local _dv_deleted
    local _dv_output_file

    _dv_deleted=0
    _dv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ ${_dv_deleted} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} "deleting old parameters ..."
        cat ${_dv_output_file}
    fi

    rm -f ${_dv_output_file}
}

#------------------------------------------------------------------------------
# create new configuration
#------------------------------------------------------------------------------
create_config ()
{
    local _cc_imax
    local _cc_idx
    local _cc_active
    local _cc_nfs_hosts_n
    local _cc_jmax
    local _cc_jdx
    local _cc_nfs_option
    local _cc_nfs_map_daemon

    myecho --pack ${module_name} "updating/creating configuration ..."

    {
        #------------------------------------------------------------------------------
        printgpl -conf "nfsserver" "2004-07-28" "jed"
        #------------------------------------------------------------------------------

        if [ "${NFSSERVER_DO_DEBUG}" = "yes" -o "${NFSSERVER_DO_DEBUG}" = "no" ]
        then
            # debug active
            printvar "NFSSERVER_DO_DEBUG" "debug mode: yes or no"
            echo
        else
            # debug parameter exists but not active
            if [ -n "`grep NFSSERVER_DO_DEBUG ${source_conf}`" ]
            then
                echo "# NFSSERVER_DO_DEBUG='yes'                   # debug mode: yes or no"
                echo
            fi
        fi

        echo
        echo '#------------------------------------------------------------------------------'
        echo '# general settings'
        echo '#------------------------------------------------------------------------------'
        echo
        printvar "START_NFSSERVER"   "start NFS daemon: yes or no"
        echo
        printvar "NFSSERVER_SUPPORT_NFS3"   "support NFSv3: yes or no"
        printvar "NFSSERVER_SUPPORT_NFS4"   "support NFSv4: yes or no"
        echo
        printvar "NFSSERVER_SHARE_N" "numbers of shares to create"

        if [ ${NFSSERVER_SHARE_N} -eq 0 ]
        then
            _cc_imax=1
        else
            _cc_imax=${NFSSERVER_SHARE_N}
        fi

        _cc_idx=1
        while [ ${_cc_idx} -le ${_cc_imax} ]
        do
            eval _cc_active='$NFSSERVER_SHARE_'${_cc_idx}'_ACTIVE'
            eval _cc_nfs_hosts_n='$NFSSERVER_SHARE_'${_cc_idx}'_HOSTS_N'

            # set defaults
            if [ -z "${_cc_active}" ]
            then
                eval NFSSERVER_SHARE_${_cc_idx}_ACTIVE="no"
            fi

            if [ ${_cc_nfs_hosts_n} -eq 0 ]
            then
                _cc_jmax=1
            else
                _cc_jmax=${_cc_nfs_hosts_n}
            fi

            # write parameters
            printvar "NFSSERVER_SHARE_${_cc_idx}_ACTIVE"      "${_cc_idx}. activate share: yes or no"
            printvar "NFSSERVER_SHARE_${_cc_idx}_PATH"        "   path of the share in filesystem"
            printvar "NFSSERVER_SHARE_${_cc_idx}_NAME"        "   name of NFSv4 share only!"
            printvar "NFSSERVER_SHARE_${_cc_idx}_HOSTS_N"     "   number of allowed hosts"

            _cc_jdx=1
            while [ ${_cc_jdx} -le ${_cc_jmax} ]
            do
                eval _cc_nfs_option='$NFSSERVER_SHARE_'${_cc_idx}'_HOSTS_'${_cc_jdx}'_OPTION'
                eval _cc_nfs_map_daemon='$NFSSERVER_SHARE_'${_cc_idx}'_HOSTS_'${_cc_jdx}'_MAP_DAEMON'

                if [ -z "${_cc_nfs_map_daemon}" ]
                then
                    eval NFSSERVER_SHARE_${_cc_idx}_HOSTS_${_cc_jdx}_MAP_DAEMON="no"
                fi

                # write parameters
                printvar "NFSSERVER_SHARE_${_cc_idx}_HOSTS_${_cc_jdx}_HOST"         "   ${_cc_jdx}. access for host(s)"
                printvar "NFSSERVER_SHARE_${_cc_idx}_HOSTS_${_cc_jdx}_RW"           "      should share writeable: yes or no"
                printvar "NFSSERVER_SHARE_${_cc_idx}_HOSTS_${_cc_jdx}_ANONUSER"     "      anonymous user or ''"
                printvar "NFSSERVER_SHARE_${_cc_idx}_HOSTS_${_cc_jdx}_ANONGROUP"    "      anonymous group or ''"
                printvar "NFSSERVER_SHARE_${_cc_idx}_HOSTS_${_cc_jdx}_ROOT_SQUASH"  "      map root user/group to anonymous"
                printvar "NFSSERVER_SHARE_${_cc_idx}_HOSTS_${_cc_jdx}_ALL_SQUASH"   "      map all users/groups to anonymous"

                if [ -n "${_cc_nfs_option}" ]
                then
                    printvar "NFSSERVER_SHARE_${_cc_idx}_HOSTS_${_cc_jdx}_OPTION"       "      add additional parameters"
                fi

              # printvar "NFSSERVER_SHARE_${_cc_idx}_HOSTS_${_cc_jdx}_MAP_DAEMON"       "      turns on dynamic uid/gid mapping"

                _cc_jdx=`expr ${_cc_jdx} + 1`
            done

            if [ ${_cc_imax} -gt ${_cc_idx} ]
            then
                echo '#------------------------------------------------------------------------------'
            fi

            _cc_idx=`expr ${_cc_idx} + 1`
        done

        #------------------------------------------------------------------------------
        printend
        #------------------------------------------------------------------------------
    } > ${generate_conf}
}

#==============================================================================
# main
#==============================================================================

pgmname=`basename $0`
module_name=`basename -s .sh ${pgmname} | cut -d- -f1`

# set variables
configfile=/etc/config.d/${module_name}
installfile=/var/run/${module_name}.install

# setting defaults
source_conf=${installfile}
generate_conf=${configfile}

goflag=0
modflag=0

case "$1" in
    *-update|update )
        goflag=1
        ;;
    *-test|test )
        source_conf=${configfile}

        generate_conf=/etc/config.d/${module_name}.test
        goflag=1
        ;;
    * )
        echo
        echo "Use one of the following options:"
        echo
        echo "  nfsserver-update.sh [update] - the file ${configfile}.install will be read, the configuration will"
        echo "                                 be checked and an updated mail configuration file will be written."
        echo
        goflag=0
esac

if [ ${goflag} -eq 1 ]
then
    if [ -f ${source_conf} ]
    then
        # previous configuration file exists
        myecho --pack ${module_name} "previous configuration found ..."
        . ${source_conf}

        rename_variables
        modify_variables
        add_variables
        delete_variables

        create_config
    else
        myecho --pack ${module_name} --error "no configuration ${source_conf} found - exiting."
        myanykey --pack ${module_name}
    fi
fi

#==============================================================================
# end
#==============================================================================
exit 0
