#!/bin/sh
#----------------------------------------------------------------------------------------
# /var/install/config.d/nfsserver.sh - configuration generator script for NFS Deamon
#
# Copyright (c) 2001-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2004-07-28 jed
# Last Update:  $Id: nfsserver.sh 73445 2020-03-24 11:54:19Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------------------

# read eislib etc.
. /var/install/include/eislib
. /var/install/include/jedlib

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2>/tmp/$(basename ${0})-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

pgmname=`basename $0`
module_name=`basename -s .sh ${pgmname} | cut -d- -f1`

# set file names
configfile=/etc/config.d/nfsserver
packagefile=/var/install/packages/nfsserver
generate_exports=/etc/exports
generate_hostsallow=/etc/hosts.allow.nfsserver
generate_hostsdeny=/etc/hosts.deny.nfsserver
generate_idmapd=/etc/idmapd.conf
generate_services=/etc/services.nfsserver
error_file=/tmp/nfsserverconf_errors
default_nfs4_exportpath=/var/lib/nfs4exports
nfs4_mountbind_cmds=${default_nfs4_exportpath}/nfs4-mountbinds

# other parameters
nfsserver_version="v`grep "<version>" ${packagefile} | sed 's#<version>\(.*\)</version>#\1#'`"

### load configuration ###
. ${configfile}
chmod 600 ${configfile}

### deleting old error file ###
if [ -f ${error_file} ]
then
    rm -f ${error_file}
fi

#----------------------------------------------------------------------------------------
# sub function - get user id
#
# input:  $1 - username
# output: uid or empty
#----------------------------------------------------------------------------------------
get_uid ()
{
    ret=1
    uid=`getent passwd ${1} | cut -d: -f3`

    if [ $? -eq 0 ]
    then
        echo ${uid}
        ret=0
    else
        uid=''
    fi

    return ${ret}
}

#----------------------------------------------------------------------------------------
# sub function - get group id
#
# input:  $1 - groupname
# output: gid or empty
#----------------------------------------------------------------------------------------
get_gid ()
{
    ret=1
    gid=`getent group ${1} | cut -d: -f3`

    if [ $? -eq 0 ]
    then
        echo ${gid}
        ret=0
    else
        gid=''
    fi

    return ${ret}
}

#----------------------------------------------------------------------------------------
# sub function - add NFS option(s)
#
# input : $1 - current option string
#         $2 - option(s) to be added
#         $3 - option(s) to be searched and replaced (optional, works only with single value)
# output: updated option string
#----------------------------------------------------------------------------------------
add_nfs_option ()
{
    # remove spaces before and after comma and remove a trailing comma
    _curr_str="`echo "$1" | sed -e 's/ *, *$//' -e 's/ *, */,/'`"
    _option_str="`echo "$2" | sed -e 's/^ *, *//' -e 's/ *, */,/' -e 's/ *, *$//'`"
    _search_str="$3"

    _multi_options=1
    while [ ${_multi_options} -eq 1 ]
    do
        echo "${_option_str}" | grep -q ','

        if [ $? -eq 0 ]
        then
            # multiple options given
            _single_option="`echo "${_option_str}" | cut -d, -f1`"
            _option_str="`echo "${_option_str}" | cut -d, -f2-`"
        else
            # single option
            _single_option="${_option_str}"
            _multi_options=0
        fi


        if [ -z "${_search_str}" ]
        then
            # add new option
            echo "${_curr_str}," | grep -q "${_single_option},"

            if [ $? -ne 0 ]
            then
                # new option hasn't been found, add it
                _curr_str="${_curr_str},${_single_option}"
            fi
        else
            # search and replace options
            echo "${_curr_str}," | grep -q "${_search_str},"

            if [ $? -eq 0 ]
            then
                # searched option found, replace it
                _curr_str="`echo "${_curr_str}," | sed -e "s/${_search_str},/${_single_option},/" -e 's/,$//'`"
            else
                # searched option not found, add new option
                _curr_str="${_curr_str},${_single_option}"
            fi
        fi
    done

    # remove trailing comma
    echo "${_curr_str}" | sed 's/,$//'
}

#----------------------------------------------------------------------------------------
# create /etc/idmapd.conf file
#----------------------------------------------------------------------------------------
create_idmapd_conf ()
{
    if [ "${NFSSERVER_SUPPORT_NFS4}" = "yes" ]
    then
        myecho --pack ${module_name} "creating idmapd.conf file ..."

        {
            #----------------------------------------------------------------------------------------
            print_short_header "${generate_idmapd}" "${pgmname}" "nfsserver" "${nfsserver_version}"
            #----------------------------------------------------------------------------------------
            echo '[General]'
            echo
            # Verbosity level of debugging (Default: 0)
            echo 'Verbosity = 0'

            echo 'Pipefs-Directory = /var/lib/nfs/rpc_pipefs'

            # The local NFSv4 domain name. An NFSv4 domain is a namespace with a unique username<->UID
            # and groupname<->GID mapping (Default: Host's fully-qualified DNS domain name)
            echo "Domain = `hostname -d`"
            echo
            echo '[Mapping]'
            echo

            # Local user name to be used when a mapping cannot be completed
            # Local group name to be used when a mapping cannot be completed
            echo 'Nobody-User  = nobody'
            echo 'Nobody-Group = nogroup'
        } > ${generate_idmapd}

        # set access rights
        chown root ${generate_idmapd}
        chgrp root ${generate_idmapd}
        chmod 0640 ${generate_idmapd}
    else
        # no nfs4 support - remove file
        rm -f ${generate_idmapd}
    fi
}

#----------------------------------------------------------------------------------------
# create /etc/exports file
#----------------------------------------------------------------------------------------
create_exports ()
{
    # NFSv4 example adapted from http://www.brennan.id.au/19-Network_File_System.html#nfs4
    #
    # /var/lib/nfs4exports/SHARE-NAME
    # /var/lib/nfs4exports/SHARE-NAME/share-1
    # /var/lib/nfs4exports/SHARE-NAME/share-2
    # /var/lib/nfs4exports/SHARE-NAME/share-3
    #
    # - If NFSSERVER_SHARE_1_PATH='nfs4' has been set we assume that it is an NFSv4 root share.
    # - The name of an NSFv4 share is read from parameter NFSSERVER_SHARE_1_NAM='SHARE-NAME'.
    # - The option 'rw/ro' is set based on parameter NFSSERVER_SHARE_1_HOSTS_x_RW.
    # - The option 'sync' will be set as default if 'async' is not set.
    # - The option 'no_subtree_check' will be set as default for NFSv4 shares.
    # - The option 'no_root_squash' is set based on parameters NFSSERVER_SHARE_x_HOSTS_y_ROOT_SQUASH
    #   and NFSSERVER_SHARE_x_HOSTS_y_ALL_SQUASH. The default value is 'root_squash'.
    # - The option 'fsid=0' will automatically be set for NFSv4 root shares.

    if [ "${START_NFSSERVER}" = 'yes' -a ${NFSSERVER_SHARE_N} -gt 0 ]
    then
        myecho --pack ${module_name} "creating exports file ..."

        # remove temporary files
        rm -f ${generate_exports}.tmp
        rm -f ${nfs4_mountbind_cmds}.tmp

        if [ "${NFSSERVER_SUPPORT_NFS4}" = "yes" ]
        then
            if [ ! -d ${default_nfs4_exportpath} ]
            then
                mkdir -p ${default_nfs4_exportpath}
            else
                # delete previously created share directories when they're empty
                find ${default_nfs4_exportpath}/ -maxdepth 2 -mount -type d | \
                grep -E -v "^${default_nfs4_exportpath}[/]?$" | sort -r | while read line
                do
                    mount | grep -E -q " ${line}[/]? "

                    if [ $? -ne 0 ]
                    then
                        rmdir --ignore-fail-on-non-empty ${line}
                    fi
                done
            fi
        fi

        share_type=3
        nfs4_count=0
        nfs4_base_share=''
        idx=1
        while [ ${idx} -le ${NFSSERVER_SHARE_N} ]
        do
            error_flag=0
            eval active='$NFSSERVER_SHARE_'${idx}'_ACTIVE'

            if [ "${active}" = "yes" ]
            then
                eval nfs_path='$NFSSERVER_SHARE_'${idx}'_PATH'
                eval nfs_name='$NFSSERVER_SHARE_'${idx}'_NAME'
                eval nfs_hosts_n='$NFSSERVER_SHARE_'${idx}'_HOSTS_N'

                if [ "${nfs_path}" = "nfs4" -a -z "${nfs_name}" ]
                then
                    # set default NFSv4 root share name
                    nfs_name='SHARES'
                fi

                if [ -n "${nfs_name}" ]
                then
                    # share name has been set therefore it must be a NFSv4 share
                    share_type=4

                    if [ ${nfs4_count} -ge 1 ]
                    then
                        # increase counter for NFSv4 sub shares
                        nfs4_count=`expr ${nfs4_count} + 1`
                    fi
                    if [ "${nfs_path}" = "nfs4" ]
                    then
                        nfs4_count=1
                        nfs4_base_share="${default_nfs4_exportpath}/${nfs_name}"
                    fi
                else
                    # NFSv3 share
                    share_type=3
                fi

                # check if directory exists, if not an NFSv4 root share
                if [ ${share_type} -eq 4 -a ${nfs4_count} -gt 1 -a ! -d "${nfs_path}" ]
                then
                    echo "Error: NFSSERVER_SHARE_${idx}_PATH='${nfs_path}' - directory doesn't exist." >> ${error_file}
                    error_flag=1
                fi

                if [ ${nfs_hosts_n} -ge 1 ]
                then
                    nfs_access_str=''
                    nfs4_bind_str=''

                    jdx=1
                    while [ ${jdx} -le ${nfs_hosts_n} ]
                    do
                        eval nfs_host='$NFSSERVER_SHARE_'${idx}'_HOSTS_'${jdx}'_HOST'
                        eval nfs_rw='$NFSSERVER_SHARE_'${idx}'_HOSTS_'${jdx}'_RW'
                        eval nfs_anonuser='$NFSSERVER_SHARE_'${idx}'_HOSTS_'${jdx}'_ANONUSER'
                        eval nfs_anongroup='$NFSSERVER_SHARE_'${idx}'_HOSTS_'${jdx}'_ANONGROUP'
                        eval nfs_root_squash='$NFSSERVER_SHARE_'${idx}'_HOSTS_'${jdx}'_ROOT_SQUASH'
                        eval nfs_all_squash='$NFSSERVER_SHARE_'${idx}'_HOSTS_'${jdx}'_ALL_SQUASH'
                        eval nfs_option='$NFSSERVER_SHARE_'${idx}'_HOSTS_'${jdx}'_OPTION'
                        eval nfs_map_daemon='$NFSSERVER_SHARE_'${idx}'_HOSTS_'${jdx}'_MAP_DAEMON'

                        # 1. remove leading comma
                        # 2. remove spaces before and after commas
                        # 3. remove trailing comma
                        nfs_option="`echo "${nfs_option}" | sed -e 's/^ *, *//' -e 's/ *, */,/' -e 's/ *, *$//'`"

                        # check if 'sync' or 'async' option have been set
                        echo ",${nfs_option}," | grep -E -q ',sync,|,async,'

                        if [ $? -eq 0 ]
                        then
                            # mount option will be added a a later stage
                            nfs_tmp_str=''
                        else
                            # set default mount option
                            nfs_tmp_str='sync'
                        fi

                        echo "${nfs_host}" | grep -q '/'

                        if [ $? -eq 0 ]
                        then
                            # change writing form 192.168.1.0/24 to 192.168.1.0/255.255.255.0
                            nfs_host="`echo "${nfs_host}" | cut -d/ -f1`/`netcalc netmask ${nfs_host}`"
                        fi

                        case ${nfs_rw} in
                            yes)
                                nfs_tmp_str="`add_nfs_option "${nfs_tmp_str}" "rw"`"
                                ;;
                            no)
                                nfs_tmp_str="`add_nfs_option "${nfs_tmp_str}" "ro"`"
                                ;;
                        esac

                        if [ "${NFSSERVER_SUPPORT_NFS4}" = "yes" ]
                        then
                            # subtree_check/no_subtree_check - If only part of a volume is exported,
                            # a routine called subtree checking verifies that a file that is requested
                            # from the client is in the appropriate part of the volume. If the entire
                            # volume is exported, disabling this check will speed up transfers.
                            if [ "${nfs_path}" = "/" ]
                            then
                                nfs_tmp_str="`add_nfs_option "${nfs_tmp_str}" "no_subtree_check" "subtree_check"`"
                            else
                                nfs_tmp_str="`add_nfs_option "${nfs_tmp_str}" "subtree_check" "no_subtree_check"`"
                            fi
                        fi

                        if [ "${nfs_all_squash}" = "yes" ]
                        then
                            # all_squash - Map all uids and gids to the anonymous user.
                            # Useful for NFS-exported public FTP directories, news spool
                            # directories, etc. The opposite option is no_all_squash,
                            # which is the default setting.
                            # Example: /pub      (ro,all_squash)
                            nfs_tmp_str="`add_nfs_option "${nfs_tmp_str}" "all_squash"`"
                        else
                            # disable squashing for all accounts
                            if [ "${nfs_root_squash}" = "no" ]
                            then
                                # no_root_squash - Turn off root squashing. This option is
                                # mainly useful for diskless clients.
                                # Example: /     master(rw) trusty(rw,no_root_squash)
                                nfs_tmp_str="`add_nfs_option "${nfs_tmp_str}" "no_root_squash"`"
                          # else
                          #     nfs_tmp_str="$nfs_tmp_str,root_squash"              # default
                            fi
                        fi

                        # anonuid and anongid - These options explicitly set the uid and gid
                        # of the anonymous account. This option is primarily useful for PC/NFS
                        # clients, where you might want all requests appear to be from one user.
                        # As an example, consider the export entry for /home/joe in the example
                        # section below, which maps all requests to uid 150 (which is supposedly
                        # that of user joe).
                        # Example: /home/joe     pc001(rw,all_squash,anonuid=150,anongid=100)

                        if [ -n "${nfs_anonuser}" ]
                        then
                            # check if feature available
                            if [ "${nfs_root_squash}" = "no" -a "${nfs_all_squash}" = "no" ]
                            then
                                {
                                    echo "Warning: NFSSERVER_SHARE_'${idx}'_HOSTS_'${jdx}'_ANONUSER can only be used if"
                                    echo "         either NFSSERVER_SHARE_${idx}_HOSTS_${jdx}_ROOT_SQUASH='yes' or"
                                    echo "         NFSSERVER_SHARE_${idx}_HOSTS_${jdx}_ALL_SQUASH='yes' has been set."
                                } >> ${error_file}
                            else
                                # get uid
                                nfs_uid=`get_uid ${nfs_anonuser}`

                                if [ -n "${nfs_uid}" ]
                                then
                                    nfs_tmp_str="`add_nfs_option "${nfs_tmp_str}" "anonuid=${nfs_uid}"`"
                                else
                                    echo "Error: NFSSERVER_SHARE_${idx}_HOSTS_${jdx}_ANONUSER='${nfs_anonuser}' - user not found." >> ${error_file}
                                    error_flag=1
                                fi

                                if [ -n "${nfs_anongroup}" -a ${error_flag} -eq 0 ]
                                then
                                    # get gid
                                    nfs_gid=`get_gid ${nfs_anongroup}`

                                    if [ -n "${nfs_gid}" ]
                                    then
                                        nfs_tmp_str="`add_nfs_option "${nfs_tmp_str}" "anongid=${nfs_gid}"`"
                                    else
                                        echo "Error: NFSSERVER_SHARE_${idx}_HOSTS_${jdx}_ANONGROUP='${nfs_anongroup}'  - group not found." >> ${error_file}
                                        error_flag=1
                                    fi
                                fi
                            fi
                        fi

                        if [ ${nfs4_count} -eq 1 ]
                        then
                            # add option for NFSv4 base share
                            nfs_tmp_str="`add_nfs_option "${nfs_tmp_str}" "fsid=0"`"
                        fi

                        # add option(s)
                        if [ -n "${nfs_option}" ]
                        then
                            nfs_tmp_str="`add_nfs_option "${nfs_tmp_str}" "${nfs_option}"`"
                        fi

                        # remove leading comma at the beginning of the parameter list
                        if [ -n "${nfs_tmp_str}" ]
                        then
                            nfs_tmp_str="`echo "${nfs_tmp_str}" | sed 's/^,//'`"
                        fi

                        if [ -z "${nfs_access_str}" ]
                        then
                            # first host entry for share
                            nfs_access_str="${nfs_host}(${nfs_tmp_str})"
                        else
                            # additional host entry for share
                            nfs_access_str="${nfs_access_str} ${nfs_host}(${nfs_tmp_str})"
                        fi

                        jdx=`expr ${jdx} + 1`
                    done
                else
                    # error no access given
                    echo "Error: NFSSERVER_SHARE_${idx}_HOSTS_N has been set to '0'." >> ${error_file}
                    error_flag=1
                fi

                # print entry
                if [ ${error_flag} -eq 0 ]
                then
                    if [ ${share_type} -eq 4 ]
                    then
                        # NFSv4 share
                        if [ "${NFSSERVER_SUPPORT_NFS4}" = "yes" ]
                        then
                            if [ ! -d ${nfs4_base_share} ]
                            then
                                # create NFSv4 base share
                                mkdir -p ${nfs4_base_share}
                            fi

                            if [ ${nfs4_count} -gt 1 ]
                            then
                                # run check only for NFSv4 sub shares
                                nfs4_sub_share="${nfs4_base_share}/${nfs_name}"

                                # check if $nfs4_base_share is part of $nfs4_sub_share
                                echo "${nfs4_sub_share}" | grep -q -E "^${nfs4_base_share}/.{1,}"

                                if [ $? -eq 0 ]
                                then
                                    # create NFSv4 sub share
                                    mkdir -p ${nfs4_sub_share}

                                    #
                                    # NFSv4 mount bind
                                    #
                                    echo "${nfs_path} ${nfs4_sub_share}" >> ${nfs4_mountbind_cmds}.tmp
                                else
                                    echo "NFSv4 PATH ERROR!"
                                fi
                            fi

                            #
                            # exports entry
                            #
                            if [ ${nfs4_count} -eq 1 ]
                            then
                                # print entry - export NFSv4 base share
                                printf "${nfs4_base_share}\t${nfs_access_str}\n" >> ${generate_exports}.tmp
                            elif [ ${nfs4_count} -gt 1 ]
                            then
                                # print entry - export NFSv4 sub shares
                                printf "${nfs4_sub_share}\t${nfs_access_str}\n" >> ${generate_exports}.tmp
                            fi
                        fi
                    else
                        # NFSv3 share
                        if [ "${NFSSERVER_SUPPORT_NFS3}" = "yes" ]
                        then
                            # print entry - export NFSv3 path
                            printf "${nfs_path}\t${nfs_access_str}\n" >> ${generate_exports}.tmp
                        fi
                    fi
                else
                    if [ ${share_type} -eq 4 ]
                    then
                        # NFSv4 share
                        if [ "${NFSSERVER_SUPPORT_NFS4}" = "yes" ]
                        then
                            echo "Error: Couldn't create NFSv4 share '${nfs4_sub_share}'! Please check the NFSSERVER_SHARE_${idx}_.. configuration." >> ${error_file}
                        fi
                    else
                        # NFSv3 share
                        if [ "${NFSSERVER_SUPPORT_NFS3}" = "yes" ]
                        then
                            echo "Error: Couldn't create NFSv3 share '${nfs_path}'! Please check the NFSSERVER_SHARE_${idx}_.. configuration." >> ${error_file}
                        fi
                    fi
                fi
            fi

            idx=`expr ${idx} + 1`
        done

        if [ -s ${generate_exports}.tmp ]
        then
            # sort exports file
            {
                #---------------------------------------------------------------------------------------
                print_short_header "${generate_exports}" "${pgmname}" "nfsserver" "${nfsserver_version}"
                #---------------------------------------------------------------------------------------
                sort ${generate_exports}.tmp
            } > ${generate_exports}
            rm -f ${generate_exports}.tmp

            # set access rights
            chown root ${generate_exports}
            chgrp root ${generate_exports}
            chmod 0640 ${generate_exports}
        fi

        if [ -s ${nfs4_mountbind_cmds}.tmp ]
        then
            # sort mount binds
            {
                #---------------------------------------------------------------------------------------
                print_short_header "${generate_exports}" "${pgmname}" "nfsserver" "${nfsserver_version}"
                #---------------------------------------------------------------------------------------
                sort ${nfs4_mountbind_cmds}.tmp
            } > ${nfs4_mountbind_cmds}
            rm -f ${nfs4_mountbind_cmds}.tmp

            # set access rights
            chown root ${nfs4_mountbind_cmds}
            chgrp root ${nfs4_mountbind_cmds}
            chmod 0640 ${nfs4_mountbind_cmds}
        fi
    else
        # no shares given - remove file
        rm -f ${generate_exports} ${generate_exports}.tmp
        rm -f ${nfs4_mountbind_cmds} ${nfs4_mountbind_cmds}.tmp
    fi
}

#----------------------------------------------------------------------------------------
# create hosts.allow file
#----------------------------------------------------------------------------------------
create_hosts_allow ()
{
    if [ "${NFSSERVER_SHARE_N}" -gt 0 ]
    then
        myecho --pack ${module_name} "creating hosts.allow file ..."

        hosts_list='127.0.0.1'

        idx=1
        while [ ${idx} -le ${NFSSERVER_SHARE_N} ]
        do
            # process all shares
            eval nfs_hosts_n='$NFSSERVER_SHARE_'${idx}'_HOSTS_N'

            jdx=1
            while [ ${jdx} -le ${nfs_hosts_n} ]
            do
                eval active='$NFSSERVER_SHARE_'${idx}'_ACTIVE'

                if [ "${active}" = 'yes' ]
                then
                    # process all hosts of a share
                    eval nfs_host='$NFSSERVER_SHARE_'${idx}'_HOSTS_'${jdx}'_HOST'

                    echo "${nfs_host}" | grep -q '/'

                    if [ $? -eq 0 ]
                    then
                        # change writing form 192.168.1.0/24 to 192.168.1.0/255.255.255.0
                        nfs_host="`echo "${nfs_host}" | cut -d/ -f1`/`netcalc netmask ${nfs_host}`"
                    fi

                    echo "${hosts_list}" | grep -q "${nfs_host}"

                    if [ $? -ne 0 ]
                    then
                        # add host to list
                        if [ -z "${hosts_list}" ]
                        then
                            hosts_list="${nfs_host}"
                        else
                            hosts_list="${hosts_list},${nfs_host}"
                        fi
                    fi
                fi

                jdx=`expr ${jdx} + 1`
            done

            idx=`expr ${idx} + 1`
        done

        # create file
        if [ -n "${hosts_list}" ]
        then
            {
                echo "#"
                echo "# ${generate_hostsallow} list file generated by ${pgmname} ${nfsserver_version}"
                echo "#"
                echo "rpcbind: ${hosts_list}"

                if [ "${NFSSERVER_SUPPORT_NFS4}" = "yes" ]
                then
                    echo "idmapd:  ${hosts_list}"
                fi

                echo "mountd:  ${hosts_list}"
                echo "nfsd:    ${hosts_list}"

                if [ "${NFSSERVER_SUPPORT_NFS4}" = "yes" ]
                then
                    echo "nfsd4:   ${hosts_list}"
                fi

                echo "statd:   ${hosts_list}"
            } > ${generate_hostsallow}

            # set access rights
            chown root ${generate_hostsallow}
            chgrp root ${generate_hostsallow}
            chmod 0640 ${generate_hostsallow}
        fi

        # update file
        /var/install/bin/update-hosts.allow ${pgmname}
    else
        # no shares given - remove file
        rm -f ${generate_hostsallow}

        # update file
        /var/install/bin/update-hosts.allow ${pgmname}
    fi
}

#----------------------------------------------------------------------------------------
# create hosts.deny file
#----------------------------------------------------------------------------------------
create_hosts_deny ()
{
    if [ "${NFSSERVER_SHARE_N}" -gt 0 ]
    then
        myecho --pack ${module_name} "creating hosts.deny file ..."

        # create file
        {
            echo "#"
            echo "# ${generate_hostsdeny} list file generated by ${pgmname} ${nfsserver_version}"
            echo "#"
            echo "rpcbind: ALL"

            if [ "${NFSSERVER_SUPPORT_NFS4}" = "yes" ]
            then
                echo "idmapd:  ALL"
            fi

            echo "mountd:  ALL"
            echo "nfsd:    ALL"

            if [ "${NFSSERVER_SUPPORT_NFS4}" = "yes" ]
            then
                echo "nfsd4:   ALL"
            fi

            echo "statd:   ALL"
        } > ${generate_hostsdeny}

        # set access rights
        chown root ${generate_hostsdeny}
        chgrp root ${generate_hostsdeny}
        chmod 0640 ${generate_hostsdeny}

        # update file
        /var/install/bin/update-hosts.deny ${pgmname}
    else
        # no shares given - remove file
        rm -f ${generate_hostsdeny}

        # update file
        /var/install/bin/update-hosts.deny ${pgmname}
    fi
}

#----------------------------------------------------------------------------------------
# create services
#----------------------------------------------------------------------------------------
create_services ()
{
    if [ "${START_NFSSERVER}" = "yes" ]
    then
        myecho --pack ${module_name} "creating services file ..."

        {
            echo '#'
            echo "# ${generate_services} list file generated by ${pgmname} ${nfsserver_version}"
            echo '#'
            printf "nfs\t2049/tcp\t\t# Network File System\n"
            printf "nfs\t2049/udp\t\t# Network File System\n"
        } > ${generate_services}

        # update file
        /var/install/bin/update-services ${pgmname}
    fi
}

#----------------------------------------------------------------------------------------
# display error file if exist
#----------------------------------------------------------------------------------------
display_error_file ()
{
    if [ -f ${error_file} ]
    then
        sleep 5
        clrhome
        myecho --pack ${module_name} --info "Configuration messages appeared, please check configuration"
        myecho --pack ${module_name}
        more ${error_file}
        myecho --pack ${module_name}
    fi
}

#========================================================================================
# main
#========================================================================================

force_quiet_run=false
if [ "$1" = '-quiet' -o "$1" = '--quiet' ]
then
    force_quiet_run=true
    shift
fi

create_idmapd_conf
create_exports
create_hosts_allow
create_hosts_deny
create_services

if ! called_from_install --pack "${module_name}"
then
    display_error_file
fi

#========================================================================================
# end
#========================================================================================
exit 0
